#include <windows.h>
#include <commctrl.h>
#include <assert.h>
#include <stdio.h>
#include "resource.h" 
#include "rules_int.h"
#include "ai_int.h"
#include "pdn.h"

#define long2point(l, pt) ((pt).x = (SHORT)LOWORD(l), (pt).y = (SHORT)HIWORD(l))
#define computer_onmove(onmove, white, black) (   ( ((onmove)== 1) && ((white)>0) ) \
                                                ||( ((onmove)==-1) && ((black)>0) )   )

/* GLOBAL VARIABLES: */
Position position;
struct{
	int sq_side;
	int left;
	int top;
} board_measurements;
int white=0, black=1; // values: human 0 | easy 1 | normal 2 | hard 3

/* FUNCTION PROTOTYPES: */
void paint_board(HDC hdc); 
void hndl_button_up(LPARAM lParam, HWND hWnd);
Sq_coords get_square_coords(POINT pt);
void get_board_measurements(HWND hWnd);
void paint_square(HDC hdc, int i, int j, Piece piece);
void paint_move(Move move, HWND hWnd);
LPTSTR GetOpenGameName(HWND hWnd);
LPTSTR GetSaveGameName(HWND hWnd);
int check_player_skill(Player player, int skill, HWND hWnd);
int uncheck_player_skill(Player player, int skill, HWND hWnd);

LRESULT WINAPI AboutDlgProc( HWND, UINT, WPARAM, LPARAM );

LRESULT CALLBACK MyWndProc(HWND hwnd, 
						   UINT message, 
						   WPARAM wParam, 
						   LPARAM lParam);
WNDCLASSEX myWinClass;
HWND hWndStatusBar;
/* FUNCTIONS: */
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
//	WNDCLASSEX myWinClass;
	HWND hMyWin, hStatus;
	MSG msg;

	/* inicializace struktury myWinClass 
	 * - urceni vlastnosti sablony okna */
	myWinClass.cbSize= sizeof(WNDCLASSEX);
	myWinClass.lpszClassName= "myWinClass";
	myWinClass.style= CS_HREDRAW | CS_VREDRAW;
	myWinClass.lpfnWndProc= MyWndProc;
	myWinClass.hInstance= hInstance;
	myWinClass.lpszMenuName= (LPCSTR)IDR_MENU1;
	myWinClass.hCursor= LoadCursor(NULL, IDC_ARROW);
	myWinClass.hIcon= LoadIcon(NULL, IDI_APPLICATION);
	myWinClass.hIconSm= 0;
	myWinClass.hbrBackground= (HBRUSH) GetStockObject(WHITE_BRUSH);
	myWinClass.cbClsExtra= 0;
	myWinClass.cbWndExtra= 0;

	/* registrace - vytvoreni "sablony okna" */
	if ( !RegisterClassEx(&myWinClass) ){
		MessageBox(NULL, "Chyba pri registraci okna", 
		           "Chyba", MB_ICONERROR);
		return 0;
	}
	
	/* vytvoreni okna */
	hMyWin= CreateWindowEx(0, "myWinClass", "Checkers", 
	                       WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, 
						   CW_USEDEFAULT, CW_USEDEFAULT, 
						   CW_USEDEFAULT, NULL, NULL /*menu*/, hInstance, 0);

	ShowWindow(hStatus, nCmdShow);
	UpdateWindow(hStatus);

	/* show & update */
	ShowWindow(hMyWin, nCmdShow);
	UpdateWindow(hMyWin);

	/* hlavni cyklus aplikace */
	while (GetMessage(&msg, NULL, 0, 0)){
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	
	return msg.wParam;
} /* end of WinMain() */

LRESULT CALLBACK MyWndProc(HWND hWnd, 
						   UINT message, 
						   WPARAM wParam, 
						   LPARAM lParam)
{
	int wmEvent;
	RECT rt;
	PAINTSTRUCT ps;
	POINT pt;
	HDC hdc;
    LPTSTR lpszFileName;
	static int lbuttondown;
	static Sq_coords chosen;

	switch(message){
		case WM_CREATE:
            hWndStatusBar = CreateWindowEx(
				0,
				STATUSCLASSNAME,
				NULL,
				WS_CHILD | WS_VISIBLE | SBARS_SIZEGRIP,
				0,
				0,
				0,
				0,
				hWnd,
				(HMENU)IDC_STATUSBAR,
				(HINSTANCE)GetWindowLong(hWnd, GWL_HINSTANCE),
				NULL);
			if (!hWndStatusBar){
				MessageBox(NULL, "Failed To Create The Status Bar", "Error", MB_OK | MB_ICONERROR);
				return 0;
			}
			SendMessage(hWndStatusBar, SB_SETTEXT, 0,(LPARAM)"Choose start square.");
			lbuttondown= 0;
			initialize_board(&position);
			chosen.x= -1;
			return 0;

		case WM_SIZE:
			SendMessage(hWndStatusBar, WM_SIZE, 0, 0);
			return 0;

		case WM_PAINT:
			hdc= BeginPaint(hWnd, &ps);
			get_board_measurements(hWnd);
			paint_board(hdc);
			EndPaint(hWnd, &ps);
			return 0;
		case WM_CLOSE:
			DestroyWindow(hWnd);
			return 0;
		case WM_DESTROY:
			PostQuitMessage(0);
			return 0;
		case WM_COMMAND:
			wmEvent= LOWORD(wParam);
			switch(wmEvent){
				case ID_GAME_EXIT: 
					DestroyWindow(hWnd);
					return 0;
				case ID_GAME_NEW:
					initialize_board(&position);
					chosen.x= -1;
					GetClientRect(hWnd, &rt);
					InvalidateRect(hWnd, &rt, 1);
					SendMessage(hWndStatusBar, SB_SETTEXT, 0,(LPARAM)"Choose start square.");
					return 0;
				case ID_PLAYERS_WHITE_HUMAN:
					uncheck_player_skill(WHITE, white, hWnd);
					white= 0;
					check_player_skill(WHITE,white,hWnd);
					break;
				case ID_PLAYERS_WHITE_EASY:
					uncheck_player_skill(WHITE, white, hWnd);
					white= 1;
					check_player_skill(WHITE,white,hWnd);
					break;
				case ID_PLAYERS_WHITE_NORMAL:
					uncheck_player_skill(WHITE, white, hWnd);
					white= 2;
					check_player_skill(WHITE,white,hWnd);
					break;
				case ID_PLAYERS_WHITE_HARD:
					uncheck_player_skill(WHITE, white, hWnd);
					white= 3;
					check_player_skill(WHITE,white,hWnd);
					break;
				case ID_PLAYERS_BLACK_HUMAN:
					uncheck_player_skill(BLACK, black, hWnd);
					black= 0;
					check_player_skill(BLACK,black,hWnd);
					break;
				case ID_PLAYERS_BLACK_EASY:
					uncheck_player_skill(BLACK, black, hWnd);
					black= 1;
					check_player_skill(BLACK,black,hWnd);
					break;
				case ID_PLAYERS_BLACK_NORMAL:
					uncheck_player_skill(BLACK, black, hWnd);
					black= 2;
					check_player_skill(BLACK,black,hWnd);
					break;
				case ID_PLAYERS_BLACK_HARD:
					uncheck_player_skill(BLACK, black, hWnd);
					black= 3;
					check_player_skill(BLACK,black,hWnd);
					break;
				case ID_GAME_SAVE:
					lpszFileName= GetSaveGameName(hWnd);
					if (!lpszFileName) break;
					history2pdn(lpszFileName);
					return 0;
				case ID_GAME_LOAD:
					lpszFileName= GetOpenGameName(hWnd);
					if (!lpszFileName) break;
					pdn2history(lpszFileName);
					load_from_history(&position);
					GetClientRect(hWnd, &rt);
					InvalidateRect(hWnd, &rt, 1);
					UpdateWindow(hWnd);
					return 0;
				case ID_HELP_ABOUT:
					DialogBox(myWinClass.hInstance, ABOUTDLG, hWnd, (DLGPROC)AboutDlgProc );
				default: 
					return DefWindowProc(hWnd, message, wParam, lParam);
			}
		case WM_LBUTTONDOWN:
			lbuttondown= 1;
			return 0;
		case WM_LBUTTONUP:
			if(!lbuttondown) return 0;
			lbuttondown= 0;
			long2point(lParam, pt);
			chosen= get_square_coords(pt);
			hndl_button_up(lParam, hWnd);
			return 0;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
	}
} /* end of MyWndProc() */

void get_board_measurements(HWND hWnd)
{
	RECT rt;
	int sq_side; /* length of one square side */
	int wh_diff; /* difference between width and height of rectangle rt */
	int left, top; /* left top corner of the board */
	
	GetClientRect(hWnd, &rt);
	/* get useful board measurements: */
	if( (rt.right - rt.left)<(rt.bottom - rt.top) ){
		sq_side= ((rt.right - rt.left) / (BOARDSIZE+2));
		wh_diff= (rt.bottom - rt.top) - BOARDSIZE * sq_side;
		left= rt.left + sq_side;
		top= rt.top + (wh_diff/2);
	}else{
		sq_side= ((rt.bottom - rt.top) / (BOARDSIZE+2));
		wh_diff= (rt.right - rt.left) - BOARDSIZE * sq_side;
		left= rt.left + (wh_diff/2);
		top= rt.top + sq_side;
	}
	board_measurements.sq_side= sq_side;
	board_measurements.left= left;
	board_measurements.top= top;
}


void paint_square(HDC hdc, int i, int j, Piece piece)
{
	int top, left, sq_side;

	top= board_measurements.top;
	left= board_measurements.left;
	sq_side= board_measurements.sq_side;

	if(is_black_square(i, BOARDSIZE-j-1))
		SelectObject(hdc, GetStockObject(LTGRAY_BRUSH));
	else
		SelectObject(hdc, GetStockObject(WHITE_BRUSH));
	Rectangle(hdc, left+i*sq_side, top+j*sq_side, 
	          left+(i+1)*sq_side+1, top+(j+1)*sq_side+1);

	switch(piece){
	case WHITE_PAWN: 
		SelectObject(hdc, GetStockObject(WHITE_BRUSH));
		Ellipse(hdc, left+i*sq_side+2, top+j*sq_side+2, 
		        left+(i+1)*sq_side-1, top+(j+1)*sq_side-1);
		break;

	case WHITE_KING: 
		SelectObject(hdc, GetStockObject(DKGRAY_BRUSH));
		Ellipse(hdc, left+i*sq_side, top+j*sq_side, 
		        left+(i+1)*sq_side, top+(j+1)*sq_side);
		SelectObject(hdc, GetStockObject(WHITE_BRUSH));
		Ellipse(hdc, left+i*sq_side+2, top+j*sq_side+2, 
		        left+(i+1)*sq_side-2, top+(j+1)*sq_side-2);
		break;

	case BLACK_PAWN: 
		SelectObject(hdc, GetStockObject(DKGRAY_BRUSH));
		Ellipse(hdc, left+i*sq_side+2, top+j*sq_side+2, 
		        left+(i+1)*sq_side-1, top+(j+1)*sq_side-1);
		break;

	case BLACK_KING: 
		SelectObject(hdc, GetStockObject(WHITE_BRUSH));
		Ellipse(hdc, left+i*sq_side, top+j*sq_side, 
		        left+(i+1)*sq_side, top+(j+1)*sq_side);
		SelectObject(hdc, GetStockObject(DKGRAY_BRUSH));
		Ellipse(hdc, left+i*sq_side+3, top+j*sq_side+3, 
		        left+(i+1)*sq_side-3, top+(j+1)*sq_side-3);
		break;
	}
}


void paint_board(HDC hdc)
{
	int i, j;

	for(i=0; i<BOARDSIZE; i++){
		for(j=0; j<BOARDSIZE; j++){
			paint_square(hdc, i, j, position.placement[i][BOARDSIZE-j-1]);
		} 
	}
}

RECT get_update_rect(int x, int y)
{
	RECT rt;
	rt.left= board_measurements.left+x*board_measurements.sq_side;
	rt.top= board_measurements.top+y*board_measurements.sq_side;
    rt.right= board_measurements.left+(x+1)*board_measurements.sq_side;
	rt.bottom= board_measurements.top+(y+1)*board_measurements.sq_side;
	return rt;
}

void blink_rect(HWND hWnd, int x, int y, Piece piece, Piece blink)
{
	RECT rt;
	rt= get_update_rect(x, BOARDSIZE-y-1);
	position.placement[x][y]= piece;
	InvalidateRect(hWnd, &rt, 1);
	UpdateWindow(hWnd);
	Sleep(200);
	position.placement[x][y]= blink;
	InvalidateRect(hWnd, &rt, 1);
	UpdateWindow(hWnd);
	Sleep(200);
	position.placement[x][y]= piece;
	InvalidateRect(hWnd, &rt, 1);
	UpdateWindow(hWnd);
	Sleep(200);
}


void paint_move(Move move, HWND hWnd)
{
	RECT rt;
	int x, y, i;
	Piece piece, blink;
	Position pos= position;

	switch(move.move_type){
		case PAWN_STEP:
		case KING_STEP:
			x= move.start.x;
			y= move.start.y;
			piece= NONE;
			blink= position.placement[x][y];
			blink_rect(hWnd, x, y, piece, blink);
			x= move.destination.x;
			y= move.destination.y;
			piece= blink;
			blink= NONE;
			blink_rect(hWnd, x, y, piece, blink);
			break;
		case KING_JUMP:
		case PAWN_JUMP:
			x= move.start.x;
			y= move.start.y;
			piece= NONE;
			blink= position.placement[x][y];
			blink_rect(hWnd, x, y, piece, blink);
			for(i=move.jump_length-1; i>0; i--){
				x= move.middle[i-1].x;
				y= move.middle[i-1].y;
				blink_rect(hWnd, x, y, piece, blink);
			}
			x= move.destination.x;
			y= move.destination.y;
			piece= blink;
			blink= NONE;
			blink_rect(hWnd, x, y, piece, blink);
			for(i=move.jump_length-1; i>=0; i--){
				x= move.captured_coords[i].x;
				y= move.captured_coords[i].y;
				position.placement[x][y]= NONE;
				rt= get_update_rect(x, BOARDSIZE-y-1);
				InvalidateRect(hWnd, &rt, 1);
			}
			UpdateWindow(hWnd);
			break;
		case PROMOTION:
			x= move.start.x;
			y= move.start.y;
			piece= NONE;
			blink= position.placement[x][y];
			blink_rect(hWnd, x, y, piece, blink);
			x= move.destination.x;
			y= move.destination.y;
			piece= promote(blink);
			blink= NONE;
			blink_rect(hWnd, x, y, piece, blink);
			break;
		case JUMP_PROMOTION:
			x= move.start.x;
			y= move.start.y;
			piece= NONE;
			blink= position.placement[x][y];
			blink_rect(hWnd, x, y, piece, blink);
			for(i=move.jump_length-1; i>0; i--){
				x= move.middle[i-1].x;
				y= move.middle[i-1].y;
				blink_rect(hWnd, x, y, piece, blink);
			}
			x= move.destination.x;
			y= move.destination.y;
			piece= promote(blink);
			blink= NONE;
			blink_rect(hWnd, x, y, piece, blink);
			for(i=move.jump_length-1; i>=0; i--){
				x= move.captured_coords[i].x;
				y= move.captured_coords[i].y;
				position.placement[x][y]= NONE;
				rt= get_update_rect(x, BOARDSIZE-y-1);
				InvalidateRect(hWnd, &rt, 1);
			}
			UpdateWindow(hWnd);
			break;
	}
	position= pos;
}


void hndl_button_up(LPARAM lParam, HWND hWnd)
{
	static Moves poss_moves;
	static Moves original_poss_moves;
	static Move move;
	static Move original_move;
	static int jump_count= 0;
	static Position pos;
	static int prvni_volani= 1;
	Game_state game_state;

	POINT pt;
	RECT clientrt;
	int i, j;
	Move mv;

	GetClientRect(hWnd, &clientrt);
	long2point(lParam, pt);

	if(move.start.x < 0 || prvni_volani){ /* choosing start square */
		prvni_volani= 0;
		move.start= get_square_coords(pt);
		if( (move.start.x >= 0) && is_black_square(move.start.x, move.start.y) ){
			pos= position;
			poss_moves= select_moves_by_start(move.start, get_all_moves(&pos));
			original_poss_moves= poss_moves;
			original_move.start= move.start;
			if(poss_moves.count == 0){
				move.start.x= -1;
				SendMessage(hWndStatusBar, SB_SETTEXT, 0,(LPARAM)"Incorrect start square, choose another.");
			}else{
				SendMessage(hWndStatusBar, SB_SETTEXT, 0,(LPARAM)"Choose destination square.");
			}
		}else{
				move.start.x= -1;
				SendMessage(hWndStatusBar, SB_SETTEXT, 0,(LPARAM)"Incorrect start square, choose another.");
		}
	
	}else{ /* start square already chosen */
		mv.start= move.start;
		mv.destination= get_square_coords(pt);
		if(poss_moves.list[0].jump_length > 1){ /* choosing middle square */
			if((i= is_member(mv.start, mv.destination, poss_moves))!=-1){ /* correct square */
				jump_count++;
				mv.move_type= PAWN_JUMP;
				mv.jump_length= 1;
				j= poss_moves.list[i].jump_length;
				mv.captured_coords[0]= poss_moves.list[i].captured_coords[j-1];
				paint_move(mv, hWnd);
				do_move(&position, mv);
				assert(!position.placement[mv.captured_coords[0].x][mv.captured_coords[0].y]);
				position.onmove*= -1;
				poss_moves= select_moves_by_start(mv.destination, get_all_moves(&position));
				move.start= mv.destination;
				original_move.middle[jump_count-1]= mv.destination;
				SendMessage(hWndStatusBar, SB_SETTEXT, 0,(LPARAM)"Enlarge the jump.");
			}else{ /* incorrect square */
				move.start.x= -1;
				SendMessage(hWndStatusBar, SB_SETTEXT, 0,(LPARAM)"Incorect move, choose start square.");
			}
		}else{ /* choosing destination square */
			if((i= is_member(mv.start, mv.destination, poss_moves))!=-1){ /* correct square */
				mv= match_move(mv, &poss_moves);
				original_move.destination= mv.destination;
				original_move= match_move(original_move, &original_poss_moves);
				paint_move(mv, hWnd);
				do_move(&position, mv);
				insert_move(original_move);
				move.start.x= -1;
				SendMessage(hWndStatusBar, SB_SETTEXT, 0,(LPARAM)"Destination chosen.");

				if((game_state= get_game_state(&position, NULL, &move)) != PROCEEDING){
					switch(game_state){
					case WHITE_WINS: 
						switch(MessageBox(hWnd, 
							              "White wins, play another?", 
										  "White wins", 
										  MB_YESNO | MB_ICONQUESTION)){
						case 6: /* YES */
							SendMessage(hWnd, WM_COMMAND, ID_GAME_NEW, 0);
							break;
						case 7: /* NO  */
							PostQuitMessage(0);
						}
						break;
					case BLACK_WINS: 
						switch(MessageBox(hWnd, 
							              "Black wins, play another?", 
										  "Black wins", 
										  MB_YESNO | MB_ICONQUESTION)){
						case 6: /* YES */
							SendMessage(hWnd, WM_COMMAND, ID_GAME_NEW, 0);
							break;
						case 7: /* NO  */
							PostQuitMessage(0);
						}
						break;
					case DRAW: 
						switch(MessageBox(hWnd, 
							              "Draw, play another?", 
										  "Draw", 
										  MB_YESNO | MB_ICONQUESTION)){
						case 6: /* YES */
							SendMessage(hWnd, WM_COMMAND, ID_GAME_NEW, 0);
							break;
						case 7: /* NO  */
							PostQuitMessage(0);
						}
						break;
					}
					return;
				}
				if(poss_moves.allocated > 0) free(poss_moves.list);
			}else{ /* incorrect square */
				move.start.x= -1;
				SendMessage(hWndStatusBar, SB_SETTEXT, 0,(LPARAM)"Incorect move, choose start square.");
			}
		}
		while(computer_onmove(position.onmove, white, black)){
			SendMessage(hWndStatusBar, SB_SETTEXT, 0,(LPARAM)"Computer thinking...");
			if (position.onmove==WHITE)
				mv= choose_best_move(&position, white);
			else
				mv= choose_best_move(&position, black);
			paint_move(mv, hWnd);
			do_move(&position, mv);
			insert_move(mv);
			if((game_state= get_game_state(&position, NULL, &move)) != PROCEEDING){
				switch(game_state){
				case WHITE_WINS: 
					switch(MessageBox(hWnd, 
						              "White wins, play another?", 
									  "White wins", 
									  MB_YESNO | MB_ICONQUESTION)){
					case 6: /* YES */
						SendMessage(hWnd, WM_COMMAND, ID_GAME_NEW, 0);
						break;
					case 7: /* NO  */
						PostQuitMessage(0);
					}
					break;
				case BLACK_WINS: 
					switch(MessageBox(hWnd, 
						              "Black wins, play another?", 
									  "Black wins", 
									  MB_YESNO | MB_ICONQUESTION)){
					case 6: /* YES */
						SendMessage(hWnd, WM_COMMAND, ID_GAME_NEW, 0);
						break;
					case 7: /* NO  */
						PostQuitMessage(0);
					}
					break;
				case DRAW: 
					switch(MessageBox(hWnd, 
						              "Draw, play another?", 
									  "Draw", 
									  MB_YESNO | MB_ICONQUESTION)){
					case 6: /* YES */
						SendMessage(hWnd, WM_COMMAND, ID_GAME_NEW, 0);
						break;
					case 7: /* NO  */
						PostQuitMessage(0);
					}
					break;
				}
				return;
			}
			SendMessage(hWndStatusBar, SB_SETTEXT, 0,(LPARAM)"Choose start square.");
		}
	}
} /* end of hndl_button_up() */

Sq_coords get_square_coords(POINT pt)
{
	Sq_coords coords;
	if( (pt.x < board_measurements.left) 
	  ||(pt.x > board_measurements.left + BOARDSIZE * board_measurements.sq_side)
	  ||(pt.y < board_measurements.top) 
	  ||(pt.y > board_measurements.top + BOARDSIZE * board_measurements.sq_side) 
	  ){
		coords.x= -1;
		coords.y= -1;
		return coords;
	}
	coords.x= (pt.x-board_measurements.left) / board_measurements.sq_side;
	coords.y= (pt.y-board_measurements.top) / board_measurements.sq_side;
	coords.y= BOARDSIZE - coords.y - 1;

	return coords;
} /* end of get_square_coords() */

int check_player_skill(int player, int skill, HWND hWnd)
{
	HMENU menu, submenu;
	int ret;

	menu= GetMenu(hWnd);
	assert(menu);
	submenu= GetSubMenu(menu,1);
	assert(submenu);
	if(player==WHITE){ 
		submenu= GetSubMenu(submenu, 0);
		assert(submenu);
	}else{ /* player==BLACK */
		submenu= GetSubMenu(submenu, 1);
		assert(submenu);
	}

	return ret= CheckMenuItem(submenu, skill, MF_BYPOSITION | MF_CHECKED); //MF_BYPOSITION |
}

int uncheck_player_skill(int player, int skill, HWND hWnd)
{
	HMENU menu, submenu;
	int ret;

	menu= GetMenu(hWnd);
	assert(menu);
	submenu= GetSubMenu(menu,1);
	assert(submenu);
	if(player==WHITE){ 
		submenu= GetSubMenu(submenu, 0);
		assert(submenu);
	}else{ /* player==BLACK */
		submenu= GetSubMenu(submenu, 1);
		assert(submenu);
	}

	return ret= CheckMenuItem(submenu, skill, MF_BYPOSITION | MF_UNCHECKED); //MF_BYPOSITION |
}


LPTSTR GetOpenGameName(HWND hWnd)
{
    // Local variables
    OPENFILENAME    OpenFileName;   // Structure for common dialog File/Open
    TCHAR           szFilter[]=__TEXT("Portable Draughts Notation(*.pdn)\0*.pdn\0All Files(*.*)\0*.*\0\0");
    TCHAR           szFile[256];
    LPTSTR          lpszFileName;

    // Initialize variables
    szFile[0] = __TEXT('\0');
    OpenFileName.lStructSize       = sizeof(OPENFILENAME);
    OpenFileName.hwndOwner         = hWnd;
//    OpenFileName.hInstance         = (HANDLE) ghInst;
    OpenFileName.lpstrFilter       = szFilter;
    OpenFileName.lpstrCustomFilter = (LPTSTR) NULL;
    OpenFileName.nMaxCustFilter    = 0L;
    OpenFileName.nFilterIndex      = 1L;
    OpenFileName.lpstrFile         = szFile;
    OpenFileName.nMaxFile          = sizeof(szFile);
    OpenFileName.lpstrFileTitle    = NULL;
    OpenFileName.nMaxFileTitle     = 0;
    OpenFileName.lpstrInitialDir   = NULL;
    OpenFileName.lpstrTitle        = __TEXT("Load Game");
    OpenFileName.nFileOffset       = 0;
    OpenFileName.nFileExtension    = 0;
    OpenFileName.lpstrDefExt       = NULL;  // No default extension
    OpenFileName.lCustData         = 0;

    OpenFileName.Flags = OFN_SHOWHELP | OFN_PATHMUSTEXIST |
                         OFN_FILEMUSTEXIST | OFN_HIDEREADONLY |
                         OFN_EXPLORER | OFN_LONGNAMES;

    if (GetOpenFileName(&OpenFileName))
    {
        // We have a valid filename, let's copy it into the buffer
        lpszFileName = GlobalAlloc(GPTR, (lstrlen(OpenFileName.lpstrFile)+1));
        if (NULL == lpszFileName)
        {
            return(NULL);
        }
        lpszFileName= strcpy(lpszFileName,OpenFileName.lpstrFile);
    }
    else // User didn't select a file
    {
        return (NULL);
    }
    return(lpszFileName);
}   // End of GetOpenGameName


LPTSTR GetSaveGameName(HWND hWnd)
{
    // Local variables
    OPENFILENAME    OpenFileName;   // Structure for common dialog File/Open
    TCHAR           szFilter[]=__TEXT("Portable Draughts Notation(*.pdn)\0*.pdn\0All Files(*.*)\0*.*\0\0");
    TCHAR           szFile[256];
    LPTSTR          lpszFileName;

    // Initialize variables
    szFile[0] = __TEXT('\0');
    OpenFileName.lStructSize       = sizeof(OPENFILENAME);
    OpenFileName.hwndOwner         = hWnd;
//    OpenFileName.hInstance         = (HANDLE) ghInst;
    OpenFileName.lpstrFilter       = szFilter;
    OpenFileName.lpstrCustomFilter = (LPTSTR) NULL;
    OpenFileName.nMaxCustFilter    = 0L;
    OpenFileName.nFilterIndex      = 1L;
    OpenFileName.lpstrFile         = szFile;
    OpenFileName.nMaxFile          = sizeof(szFile);
    OpenFileName.lpstrFileTitle    = NULL;
    OpenFileName.nMaxFileTitle     = 0;
    OpenFileName.lpstrInitialDir   = NULL;
    OpenFileName.lpstrTitle        = __TEXT("Save Game As...");
    OpenFileName.nFileOffset       = 0;
    OpenFileName.nFileExtension    = 0;
    OpenFileName.lpstrDefExt       = NULL;  // No default extension
    OpenFileName.lCustData         = 0;
	OpenFileName.lpstrDefExt       = "pdn";

    OpenFileName.Flags = OFN_SHOWHELP | OFN_PATHMUSTEXIST |
                         OFN_HIDEREADONLY |
                         OFN_EXPLORER | OFN_LONGNAMES;

    if (GetSaveFileName(&OpenFileName))
    {
        // We have a valid filename, let's copy it into the buffer
        lpszFileName = GlobalAlloc(GPTR, (lstrlen(OpenFileName.lpstrFile)+1));
        if (NULL == lpszFileName)
        {
            return(NULL);
        }
        lpszFileName= strcpy(lpszFileName,OpenFileName.lpstrFile);
    }
    else // User didn't select a file
    {
        return (NULL);
    }
    return(lpszFileName);
}   // End of GetOpenGameName

LRESULT WINAPI AboutDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch( uMsg ) {
	case WM_INITDIALOG:
		return TRUE;
	case WM_COMMAND:
		switch( wParam ) {
        case IDOK:
            EndDialog( hDlg, TRUE );
            return TRUE;
		}
	default:
		return DefWindowProc(hDlg, uMsg, wParam, lParam);
	}
return FALSE;

}

